import {
  User,
  InsertUser,
  Category,
  InsertCategory,
  Listing,
  InsertListing,
  ListingImage,
  InsertListingImage,
  Message,
  InsertMessage,
  Favorite,
  InsertFavorite,
} from "@shared/schema";

// Interface for all storage operations
export interface IStorage {
  // User operations
  getUser(id: number): Promise<User | undefined>;
  getUserByUsername(username: string): Promise<User | undefined>;
  getUserByEmail(email: string): Promise<User | undefined>;
  createUser(user: InsertUser): Promise<User>;
  updateUser(id: number, user: Partial<User>): Promise<User | undefined>;
  
  // Category operations
  getCategories(): Promise<Category[]>;
  getCategory(id: number): Promise<Category | undefined>;
  getCategoryBySlug(slug: string): Promise<Category | undefined>;
  createCategory(category: InsertCategory): Promise<Category>;
  updateCategoryListingCount(id: number, increment: boolean): Promise<Category | undefined>;
  
  // Listing operations
  getListings(options?: { 
    limit?: number, 
    offset?: number,
    categoryId?: number,
    userId?: number,
    searchTerm?: string,
    minPrice?: number,
    maxPrice?: number,
    location?: string,
    sortBy?: string,
    sortOrder?: 'asc' | 'desc'
  }): Promise<{ listings: Listing[], total: number }>;
  getListing(id: number): Promise<Listing | undefined>;
  createListing(listing: InsertListing): Promise<Listing>;
  updateListing(id: number, listing: Partial<Listing>): Promise<Listing | undefined>;
  deleteListing(id: number): Promise<boolean>;
  
  // Listing images operations
  getListingImages(listingId: number): Promise<ListingImage[]>;
  createListingImage(image: InsertListingImage): Promise<ListingImage>;
  deleteListingImage(id: number): Promise<boolean>;
  setPrimaryListingImage(listingId: number, imageId: number): Promise<boolean>;
  
  // Message operations
  getMessages(userId: number, listingId?: number): Promise<Message[]>;
  createMessage(message: InsertMessage): Promise<Message>;
  markMessageAsRead(id: number): Promise<boolean>;
  
  // Favorite operations
  getUserFavorites(userId: number): Promise<Listing[]>;
  addFavorite(favorite: InsertFavorite): Promise<Favorite>;
  removeFavorite(userId: number, listingId: number): Promise<boolean>;
  isFavorite(userId: number, listingId: number): Promise<boolean>;
}

// In-memory storage implementation
export class MemStorage implements IStorage {
  private users: Map<number, User>;
  private categories: Map<number, Category>;
  private listings: Map<number, Listing>;
  private listingImages: Map<number, ListingImage>;
  private messages: Map<number, Message>;
  private favorites: Map<number, Favorite>;
  
  private userIdCounter: number;
  private categoryIdCounter: number;
  private listingIdCounter: number;
  private listingImageIdCounter: number;
  private messageIdCounter: number;
  private favoriteIdCounter: number;

  constructor() {
    this.users = new Map();
    this.categories = new Map();
    this.listings = new Map();
    this.listingImages = new Map();
    this.messages = new Map();
    this.favorites = new Map();
    
    this.userIdCounter = 1;
    this.categoryIdCounter = 1;
    this.listingIdCounter = 1;
    this.listingImageIdCounter = 1;
    this.messageIdCounter = 1;
    this.favoriteIdCounter = 1;
    
    // Initialize with some categories
    this.initializeCategories();
  }

  // Initialize default categories
  private initializeCategories() {
    const defaultCategories: InsertCategory[] = [
      { name: "Véhicules", slug: "vehicules", icon: "fa-car", listingCount: 0 },
      { name: "Immobilier", slug: "immobilier", icon: "fa-home", listingCount: 0 },
      { name: "Meubles", slug: "meubles", icon: "fa-couch", listingCount: 0 },
      { name: "Vêtements", slug: "vetements", icon: "fa-tshirt", listingCount: 0 },
      { name: "Multimédia", slug: "multimedia", icon: "fa-laptop", listingCount: 0 },
      { name: "Loisirs", slug: "loisirs", icon: "fa-volleyball-ball", listingCount: 0 },
      { name: "Bricolage", slug: "bricolage", icon: "fa-tools", listingCount: 0 },
      { name: "Emploi", slug: "emploi", icon: "fa-briefcase", listingCount: 0 },
      { name: "Vacances", slug: "vacances", icon: "fa-palm-tree", listingCount: 0 },
    ];

    defaultCategories.forEach(category => {
      this.createCategory(category);
    });
  }

  // User operations
  async getUser(id: number): Promise<User | undefined> {
    return this.users.get(id);
  }

  async getUserByUsername(username: string): Promise<User | undefined> {
    return Array.from(this.users.values()).find(user => 
      user.username.toLowerCase() === username.toLowerCase()
    );
  }

  async getUserByEmail(email: string): Promise<User | undefined> {
    return Array.from(this.users.values()).find(user => 
      user.email.toLowerCase() === email.toLowerCase()
    );
  }

  async createUser(user: InsertUser): Promise<User> {
    const id = this.userIdCounter++;
    const now = new Date();
    const newUser: User = { ...user, id, createdAt: now };
    this.users.set(id, newUser);
    return newUser;
  }

  async updateUser(id: number, userData: Partial<User>): Promise<User | undefined> {
    const user = this.users.get(id);
    if (!user) return undefined;
    
    const updatedUser: User = { ...user, ...userData };
    this.users.set(id, updatedUser);
    return updatedUser;
  }

  // Category operations
  async getCategories(): Promise<Category[]> {
    return Array.from(this.categories.values());
  }

  async getCategory(id: number): Promise<Category | undefined> {
    return this.categories.get(id);
  }

  async getCategoryBySlug(slug: string): Promise<Category | undefined> {
    return Array.from(this.categories.values()).find(category => 
      category.slug === slug
    );
  }

  async createCategory(category: InsertCategory): Promise<Category> {
    const id = this.categoryIdCounter++;
    const newCategory: Category = { ...category, id };
    this.categories.set(id, newCategory);
    return newCategory;
  }

  async updateCategoryListingCount(id: number, increment: boolean): Promise<Category | undefined> {
    const category = this.categories.get(id);
    if (!category) return undefined;
    
    const updatedCount = increment ? category.listingCount + 1 : Math.max(0, category.listingCount - 1);
    const updatedCategory: Category = { ...category, listingCount: updatedCount };
    this.categories.set(id, updatedCategory);
    return updatedCategory;
  }

  // Listing operations
  async getListings(options: { 
    limit?: number, 
    offset?: number,
    categoryId?: number,
    userId?: number,
    searchTerm?: string,
    minPrice?: number,
    maxPrice?: number,
    location?: string,
    sortBy?: string,
    sortOrder?: 'asc' | 'desc'
  } = {}): Promise<{ listings: Listing[], total: number }> {
    let filteredListings = Array.from(this.listings.values());
    
    // Filter by category
    if (options.categoryId) {
      filteredListings = filteredListings.filter(listing => 
        listing.categoryId === options.categoryId
      );
    }
    
    // Filter by user
    if (options.userId) {
      filteredListings = filteredListings.filter(listing => 
        listing.userId === options.userId
      );
    }
    
    // Filter by search term (in title and description)
    if (options.searchTerm) {
      const term = options.searchTerm.toLowerCase();
      filteredListings = filteredListings.filter(listing => 
        listing.title.toLowerCase().includes(term) || listing.description.toLowerCase().includes(term)
      );
    }
    
    // Filter by price range
    if (options.minPrice !== undefined) {
      filteredListings = filteredListings.filter(listing => 
        listing.price !== null && listing.price >= options.minPrice!
      );
    }
    
    if (options.maxPrice !== undefined) {
      filteredListings = filteredListings.filter(listing => 
        listing.price !== null && listing.price <= options.maxPrice!
      );
    }
    
    // Filter by location
    if (options.location) {
      filteredListings = filteredListings.filter(listing => 
        listing.location.toLowerCase().includes(options.location!.toLowerCase())
      );
    }
    
    // Sort listings
    if (options.sortBy) {
      const sortOrder = options.sortOrder === 'asc' ? 1 : -1;
      
      filteredListings.sort((a, b) => {
        switch (options.sortBy) {
          case 'price':
            return ((a.price || 0) - (b.price || 0)) * sortOrder;
          case 'date':
            return (new Date(a.createdAt).getTime() - new Date(b.createdAt).getTime()) * sortOrder;
          default:
            return 0;
        }
      });
    } else {
      // Default sort by newest first
      filteredListings.sort((a, b) => 
        new Date(b.createdAt).getTime() - new Date(a.createdAt).getTime()
      );
    }
    
    // Pagination
    const total = filteredListings.length;
    
    if (options.limit !== undefined && options.offset !== undefined) {
      filteredListings = filteredListings.slice(options.offset, options.offset + options.limit);
    }
    
    return { listings: filteredListings, total };
  }

  async getListing(id: number): Promise<Listing | undefined> {
    return this.listings.get(id);
  }

  async createListing(listing: InsertListing): Promise<Listing> {
    const id = this.listingIdCounter++;
    const now = new Date();
    const newListing: Listing = { ...listing, id, createdAt: now };
    this.listings.set(id, newListing);
    
    // Update category listing count
    await this.updateCategoryListingCount(listing.categoryId, true);
    
    return newListing;
  }

  async updateListing(id: number, listingData: Partial<Listing>): Promise<Listing | undefined> {
    const listing = this.listings.get(id);
    if (!listing) return undefined;
    
    const updatedListing: Listing = { ...listing, ...listingData };
    this.listings.set(id, updatedListing);
    
    // If the category changed, update the counts
    if (listingData.categoryId && listingData.categoryId !== listing.categoryId) {
      await this.updateCategoryListingCount(listing.categoryId, false);
      await this.updateCategoryListingCount(listingData.categoryId, true);
    }
    
    return updatedListing;
  }

  async deleteListing(id: number): Promise<boolean> {
    const listing = this.listings.get(id);
    if (!listing) return false;
    
    this.listings.delete(id);
    
    // Update category listing count
    await this.updateCategoryListingCount(listing.categoryId, false);
    
    // Delete all images associated with the listing
    Array.from(this.listingImages.values())
      .filter(image => image.listingId === id)
      .forEach(image => this.listingImages.delete(image.id));
    
    // Delete all favorites associated with the listing
    Array.from(this.favorites.values())
      .filter(favorite => favorite.listingId === id)
      .forEach(favorite => this.favorites.delete(favorite.id));
    
    return true;
  }

  // Listing images operations
  async getListingImages(listingId: number): Promise<ListingImage[]> {
    return Array.from(this.listingImages.values())
      .filter(image => image.listingId === listingId);
  }

  async createListingImage(image: InsertListingImage): Promise<ListingImage> {
    const id = this.listingImageIdCounter++;
    const newImage: ListingImage = { ...image, id };
    this.listingImages.set(id, newImage);
    
    // If this is the first image for the listing, make it primary
    const existingImages = await this.getListingImages(image.listingId);
    if (existingImages.length === 1) {
      await this.setPrimaryListingImage(image.listingId, id);
    }
    
    return newImage;
  }

  async deleteListingImage(id: number): Promise<boolean> {
    const image = this.listingImages.get(id);
    if (!image) return false;
    
    this.listingImages.delete(id);
    
    // If this was the primary image, set another image as primary
    if (image.isPrimary) {
      const remainingImages = await this.getListingImages(image.listingId);
      if (remainingImages.length > 0) {
        await this.setPrimaryListingImage(image.listingId, remainingImages[0].id);
      }
    }
    
    return true;
  }

  async setPrimaryListingImage(listingId: number, imageId: number): Promise<boolean> {
    // Reset all images for this listing to non-primary
    Array.from(this.listingImages.values())
      .filter(image => image.listingId === listingId)
      .forEach(image => {
        const updated = { ...image, isPrimary: false };
        this.listingImages.set(image.id, updated);
      });
    
    // Set the selected image as primary
    const image = this.listingImages.get(imageId);
    if (!image || image.listingId !== listingId) return false;
    
    const updatedImage = { ...image, isPrimary: true };
    this.listingImages.set(imageId, updatedImage);
    
    return true;
  }

  // Message operations
  async getMessages(userId: number, listingId?: number): Promise<Message[]> {
    return Array.from(this.messages.values())
      .filter(message => 
        (message.senderId === userId || message.receiverId === userId) &&
        (listingId === undefined || message.listingId === listingId)
      )
      .sort((a, b) => new Date(a.createdAt).getTime() - new Date(b.createdAt).getTime());
  }

  async createMessage(message: InsertMessage): Promise<Message> {
    const id = this.messageIdCounter++;
    const now = new Date();
    const newMessage: Message = { ...message, id, createdAt: now };
    this.messages.set(id, newMessage);
    return newMessage;
  }

  async markMessageAsRead(id: number): Promise<boolean> {
    const message = this.messages.get(id);
    if (!message) return false;
    
    const updatedMessage: Message = { ...message, isRead: true };
    this.messages.set(id, updatedMessage);
    return true;
  }

  // Favorite operations
  async getUserFavorites(userId: number): Promise<Listing[]> {
    const favoriteListingIds = Array.from(this.favorites.values())
      .filter(favorite => favorite.userId === userId)
      .map(favorite => favorite.listingId);
    
    return Array.from(this.listings.values())
      .filter(listing => favoriteListingIds.includes(listing.id));
  }

  async addFavorite(favorite: InsertFavorite): Promise<Favorite> {
    // Check if already a favorite
    const existing = Array.from(this.favorites.values()).find(f => 
      f.userId === favorite.userId && f.listingId === favorite.listingId
    );
    
    if (existing) return existing;
    
    const id = this.favoriteIdCounter++;
    const now = new Date();
    const newFavorite: Favorite = { ...favorite, id, createdAt: now };
    this.favorites.set(id, newFavorite);
    return newFavorite;
  }

  async removeFavorite(userId: number, listingId: number): Promise<boolean> {
    const favorite = Array.from(this.favorites.values()).find(f => 
      f.userId === userId && f.listingId === listingId
    );
    
    if (!favorite) return false;
    
    this.favorites.delete(favorite.id);
    return true;
  }

  async isFavorite(userId: number, listingId: number): Promise<boolean> {
    return Array.from(this.favorites.values()).some(f => 
      f.userId === userId && f.listingId === listingId
    );
  }
}

export const storage = new MemStorage();
